@@ -187,6 +187,15 @@ int hwloc_internal_distances_dup(struct hwloc_topology *new, struct hwloc_topolo
 {
   struct hwloc_internal_distances_s *olddist;
   int err;
+
+  new->grouping = old->grouping;
+  if (old->grouping) {
+    new->grouping_verbose = old->grouping_verbose;
+    new->grouping_nbaccuracies = old->grouping_nbaccuracies;
+    memcpy(new->grouping_accuracies, old->grouping_accuracies, sizeof(old->grouping_accuracies));
+    new->grouping_next_subkind = old->grouping_next_subkind;
+  }
+
   new->next_dist_id = old->next_dist_id;
   for(olddist = old->first_dist; olddist; olddist = olddist->next) {
     err = hwloc_internal_distances_dup_one(new, olddist);
@@ -1037,7 +1046,7 @@ hwloc_distances_get_by_name(hwloc_topology_t topology, const char *name,
     return -1;
   }
 
-  return hwloc__distances_get(topology, name, HWLOC_OBJ_TYPE_NONE, nrp, distancesp, HWLOC_DISTANCES_KIND_ALL, flags);
+  return hwloc__distances_get(topology, name, HWLOC_OBJ_TYPE_NONE, nrp, distancesp, 0, flags);
 }
 
 int
