@@ -379,6 +379,7 @@ to_internal_location(struct hwloc_internal_location_s *iloc,
       errno = EINVAL;
       return -1;
     }
+    iloc->location.object.obj = location->location.object;
     iloc->location.object.gp_index = location->location.object->gp_index;
     iloc->location.object.type = location->location.object->type;
     return 0;
@@ -1158,6 +1159,9 @@ match_local_obj_cpuset(hwloc_obj_t node, hwloc_cpuset_t cpuset, unsigned long fl
 {
   if (flags & HWLOC_LOCAL_NUMANODE_FLAG_ALL)
     return 1;
+  if ((flags & HWLOC_LOCAL_NUMANODE_FLAG_INTERSECT_LOCALITY)
+      && hwloc_bitmap_intersects(node->cpuset, cpuset))
+    return 1;
   if ((flags & HWLOC_LOCAL_NUMANODE_FLAG_LARGER_LOCALITY)
       && hwloc_bitmap_isincluded(cpuset, node->cpuset))
     return 1;
@@ -1180,6 +1184,7 @@ hwloc_get_local_numanode_objs(hwloc_topology_t topology,
 
   if (flags & ~(HWLOC_LOCAL_NUMANODE_FLAG_SMALLER_LOCALITY
                 |HWLOC_LOCAL_NUMANODE_FLAG_LARGER_LOCALITY
+                |HWLOC_LOCAL_NUMANODE_FLAG_INTERSECT_LOCALITY
                 | HWLOC_LOCAL_NUMANODE_FLAG_ALL)) {
     errno = EINVAL;
     return -1;
@@ -1520,10 +1525,15 @@ hwloc__group_memory_tiers(hwloc_topology_t topology,
     }
   }
 
-  /* Sort nodes.
-   * We could also sort by the existing subtype.
-   * KNL is the only case where subtypes are set in backends, but we set memattrs as well there.
-   * Also HWLOC_MEMTIERS_REFRESH would be a special value to ignore existing subtypes.
+  /* Sort nodes by tier type and bandwidth.
+   *
+   * We could also use the existing subtype but it's not clear it'd be better.
+   * For NVIDIA GPU, "GPUMemory" is set in the Linux backend, and used above to set tier type anyway.
+   * For KNL, the Linux backend sets subtypes and memattrs, sorting by memattrs already works fine.
+   * Existing subtypes could have been imported from XML, usually mostly OK except maybe SPM (fallback for I don't know)?
+   * An envvar (or HWLOC_MEMTIERS_REFRESH special value?) could be passed to ignore existing subtypes,
+   * but "GPUMemory" wouldn't be available anymore, we'd have to use something else like "PCIBusId",
+   * but that one might not always be specific to GPU-backed NUMA nodes?
    */
   hwloc_debug("Sorting memory node infos...\n");
   qsort(nodeinfos, n, sizeof(*nodeinfos), compare_node_infos_by_type_and_bw);
