@@ -1,17 +1,16 @@
 /*
  * Copyright © 2009 CNRS
- * Copyright © 2009-2019 Inria.  All rights reserved.
+ * Copyright © 2009-2025 Inria.  All rights reserved.
  * Copyright © 2009-2010, 2012 Université Bordeaux
  * Copyright © 2009-2018 Cisco Systems, Inc.  All rights reserved.
  * See COPYING in top-level directory.
  */
 
-#include <private/autogen/config.h>
-#include <hwloc-calc.h>
-#include <hwloc.h>
-
+#include "private/autogen/config.h"
+#include "hwloc-calc.h"
+#include "hwloc.h"
 #ifdef HWLOC_LINUX_SYS
-#include <hwloc/linux.h>
+#include "hwloc/linux.h"
 #endif
 
 #ifdef HAVE_UNISTD_H
@@ -28,15 +27,27 @@
 
 void usage(const char *name, FILE *where)
 {
-  fprintf(where, "Usage: %s [options] <location> -- command ...\n", name);
+  fprintf(where, "Usage: %s [topology options] [options] <location> -- command ...\n", name);
   fprintf(where, " <location> may be a space-separated list of cpusets or objects\n");
   fprintf(where, "            as supported by the hwloc-calc utility, e.g:\n");
   hwloc_calc_locations_usage(where);
+  fprintf(where, "Input topology options (must be at the beginning):\n");
+  fprintf(where, "  --no-smt       Only keep a single PU per core\n");
+  fprintf(where, "  --restrict [nodeset=]<bitmap>\n");
+  fprintf(where, "                 Restrict the topology to some processors or NUMA nodes.\n");
+  fprintf(where, "  --restrict-flags <n>  Set the flags to be used during restrict\n");
+  fprintf(where, "  --disallowed   Include objects disallowed by administrative limitations\n");
+  fprintf(where, "  --hbm          Only consider high bandwidth memory nodes\n");
+  fprintf(where, "  --no-hbm       Ignore high-bandwidth memory nodes\n");
   fprintf(where, "Options:\n");
   fprintf(where, "  --cpubind      Use following arguments for cpu binding (default)\n");
   fprintf(where, "  --membind      Use following arguments for memory binding\n");
-  fprintf(where, "  --mempolicy <default|firsttouch|bind|interleave|nexttouch>\n"
+  fprintf(where, "  --mempolicy <default|firsttouch|bind|interleave|weighted|nexttouch>\n"
 		 "                 Change policy that --membind applies (default is bind)\n");
+  fprintf(where, "  --best-memattr <attr>\n");
+  fprintf(where, "                 Select the best target node in the given memory binding\n");
+  fprintf(where, "  --default-nodes\n");
+  fprintf(where, "                 Keep default memory nodes only\n");
   fprintf(where, "  -l --logical   Take logical object indexes (default)\n");
   fprintf(where, "  -p --physical  Take physical object indexes\n");
   fprintf(where, "  --single       Bind on a single CPU to prevent migration\n");
@@ -49,37 +60,36 @@ void usage(const char *name, FILE *where)
 #ifdef HWLOC_LINUX_SYS
   fprintf(where, "  --tid <tid>    Operate on thread <tid>\n");
 #endif
-  fprintf(where, "  --taskset      Use taskset-specific format when displaying cpuset strings\n");
-  fprintf(where, "Input topology options:\n");
-  fprintf(where, "  --restrict <set> Restrict the topology to processors listed in <set>\n");
-  fprintf(where, "  --whole-system   Do not consider administration limitations\n");
-  fprintf(where, "  --hbm          Only consider high bandwidth memory nodes\n");
-  fprintf(where, "  --no-hbm       Ignore high-bandwidth memory nodes\n");
+  fprintf(where, "  --cpuset-output-format <hwloc|list|taskset>\n"
+                 "  --cof <hwloc|list|taskset>\n"
+                 "                 Change the format of cpuset outputs\n");
   fprintf(where, "Miscellaneous options:\n");
   fprintf(where, "  -f --force     Launch the command even if binding failed\n");
   fprintf(where, "  -q --quiet     Hide non-fatal error messages\n");
   fprintf(where, "  -v --verbose   Show verbose messages\n");
   fprintf(where, "  --version      Report version and exit\n");
+  fprintf(where, "  -h --help      Show this usage\n");
 }
 
 int main(int argc, char *argv[])
 {
   hwloc_topology_t topology;
-  int loaded = 0;
   int depth = -1;
-  hwloc_bitmap_t cpubind_set, membind_set;
+  hwloc_bitmap_t cpubind_set, membind_set, dummy_set;
   int got_cpubind = 0, got_membind = 0;
   int working_on_cpubind = 1; /* membind if 0 */
   int get_binding = 0;
   int use_nodeset = 0;
   int get_last_cpu_location = 0;
   unsigned long flags = 0;
+  unsigned long restrict_flags = 0;
   int force = 0;
   int single = 0;
   int verbose = 0;
+  int no_smt = -1;
   int only_hbm = -1;
   int logical = 1;
-  int taskset = 0;
+  enum hwloc_utils_cpuset_format_e cpuset_output_format = HWLOC_UTILS_CPUSET_FORMAT_HWLOC;
   unsigned cpubind_flags = 0;
   hwloc_membind_policy_t membind_policy = HWLOC_MEMBIND_BIND;
   int got_mempolicy = 0;
@@ -89,11 +99,21 @@ int main(int argc, char *argv[])
   int pid_number = -1;
   int tid_number = -1;
   hwloc_pid_t pid = 0; /* only valid when pid_number > 0, but gcc-4.8 still reports uninitialized warnings */
+  hwloc_memattr_id_t best_memattr_id = (hwloc_memattr_id_t) -1;
+  char *best_memattr_str = NULL;
+  int default_nodes = 0;
   char *callname;
+  char *restrictstring = NULL;
+  const char *env;
   struct hwloc_calc_location_context_s lcontext;
   struct hwloc_calc_set_context_s scontext;
 
-  callname = argv[0];
+  callname = strrchr(argv[0], '/');
+  if (!callname)
+    callname = argv[0];
+  else
+    callname++;
+
   /* skip argv[0], handle options */
   argv++;
   argc--;
@@ -102,22 +122,106 @@ int main(int argc, char *argv[])
 
   cpubind_set = hwloc_bitmap_alloc();
   membind_set = hwloc_bitmap_alloc();
-
-  /* don't load now, in case some options change the config before the topology is actually used */
-#define LOADED() (loaded)
-#define ENSURE_LOADED() do { \
-  if (!loaded) { \
-    hwloc_topology_init(&topology); \
-    hwloc_topology_set_all_types_filter(topology, HWLOC_TYPE_FILTER_KEEP_ALL); \
-    hwloc_topology_set_flags(topology, flags); \
-    ret = hwloc_topology_load(topology); \
-    if (ret < 0) return EXIT_FAILURE; \
-    depth = hwloc_topology_get_depth(topology); \
-    loaded = 1; \
-  } \
-} while (0)
+  dummy_set = hwloc_bitmap_alloc();
 
   while (argc >= 1) {
+    opt = 0;
+
+    if (!strcmp (argv[0], "--disallowed") || !strcmp (argv[0], "--whole-system")) {
+      flags |= HWLOC_TOPOLOGY_FLAG_INCLUDE_DISALLOWED;
+      goto next_config;
+    }
+    if (!strcmp (argv[0], "--restrict")) {
+      if (argc < 2) {
+        usage (callname, stderr);
+        exit(EXIT_FAILURE);
+      }
+      if(strncmp(argv[1], "nodeset=", 7))
+        restrictstring = strdup(argv[1]);
+      else {
+        restrictstring = strdup(argv[1]+8);
+        restrict_flags |= HWLOC_RESTRICT_FLAG_BYNODESET;
+      }
+      opt = 1;
+      goto next_config;
+    }
+    if (!strcmp (argv[0], "--restrict-flags")) {
+      if (argc < 2) {
+        usage (callname, stderr);
+        exit(EXIT_FAILURE);
+      }
+      restrict_flags = hwloc_utils_parse_restrict_flags(argv[1]);
+      opt = 1;
+      goto next_config;
+    }
+    if (!strcmp(argv[0], "--no-smt")) {
+      no_smt = 0;
+      goto next_config;
+    }
+    if (!strncmp(argv[0], "--no-smt=", 9)) {
+      no_smt = atoi(argv[0] + 9);
+      goto next_config;
+    }
+    if (!strcmp(argv[0], "--hbm")) {
+      only_hbm = 1;
+      goto next_config;
+    }
+    if (!strcmp(argv[0], "--no-hbm")) {
+      only_hbm = 0;
+      goto next_config;
+    }
+    if (!strcmp (argv[0], "--best-memattr")) {
+      if (argc < 2) {
+        usage (callname, stderr);
+        exit(EXIT_FAILURE);
+      }
+      best_memattr_str = argv[1];
+      opt = 1;
+      goto next_config;
+    }
+    if (!strcmp (argv[0], "--default-nodes")) {
+      default_nodes = 1;
+      goto next_config;
+    }
+
+    break;
+
+  next_config:
+    argc -= opt+1;
+    argv += opt+1;
+  }
+
+  /* show binding error messages, e.g. PREFERRED_MANY not supported by Linux kernel */
+  env = getenv("HWLOC_HIDE_ERRORS");
+  if (!env || atoi(env) != 2) {
+    if (!getenv("HWLOC_SHOW_ERRORS"))
+      putenv((char *) "HWLOC_SHOW_ERRORS=bind");
+  }
+
+  hwloc_topology_init(&topology);
+  hwloc_topology_set_all_types_filter(topology, HWLOC_TYPE_FILTER_KEEP_ALL);
+  hwloc_topology_set_flags(topology, flags);
+  ret = hwloc_topology_load(topology);
+  if (ret < 0) {
+    perror("Couldn't load the topology");
+    return EXIT_FAILURE;
+  }
+  if (restrictstring) {
+    hwloc_bitmap_t restrictset = hwloc_bitmap_alloc();
+    hwloc_bitmap_sscanf(restrictset, restrictstring);
+    if (hwloc_topology_restrict (topology, restrictset, restrict_flags)) {
+      perror("Restricting the topology");
+      /* FALLTHRU */
+    }
+    hwloc_bitmap_free(restrictset);
+    free(restrictstring);
+  }
+  depth = hwloc_topology_get_depth(topology);
+
+  while (argc >= 1) {
+    char *location;
+    int nodeset_location;
+
     if (!strcmp(argv[0], "--")) {
       argc--;
       argv++;
@@ -135,8 +239,8 @@ int main(int argc, char *argv[])
 	verbose--;
 	goto next;
       }
-      if (!strcmp(argv[0], "--help")) {
-        usage("hwloc-bind", stdout);
+      if (!strcmp(argv[0], "-h") || !strcmp(argv[0], "--help")) {
+        usage(callname, stdout);
 	return EXIT_SUCCESS;
       }
       if (!strcmp(argv[0], "--single")) {
@@ -154,7 +258,7 @@ int main(int argc, char *argv[])
       }
       if (!strcmp(argv[0], "--pid")) {
         if (argc < 2) {
-          usage ("hwloc-bind", stderr);
+          usage (callname, stderr);
           exit(EXIT_FAILURE);
         }
         pid_number = atoi(argv[1]);
@@ -164,7 +268,7 @@ int main(int argc, char *argv[])
 #ifdef HWLOC_LINUX_SYS
       if (!strcmp(argv[0], "--tid")) {
         if (argc < 2) {
-          usage ("hwloc-bind", stderr);
+          usage (callname, stderr);
           exit(EXIT_FAILURE);
         }
         tid_number = atoi(argv[1]);
@@ -184,8 +288,21 @@ int main(int argc, char *argv[])
         logical = 0;
         goto next;
       }
+      if (!strcmp(argv[0], "--cpuset-output-format") || !strcmp(argv[0], "--cof")) {
+        if (argc < 2) {
+          usage (callname, stderr);
+          exit(EXIT_FAILURE);
+        }
+        cpuset_output_format = hwloc_utils_parse_cpuset_format(argv[1]);
+        if (HWLOC_UTILS_CPUSET_FORMAT_UNKNOWN == cpuset_output_format) {
+          fprintf(stderr, "Unrecognized %s argument %s\n", argv[0], argv[1]);
+          exit(EXIT_FAILURE);
+        }
+        opt = 1;
+        goto next;
+      }
       if (!strcmp(argv[0], "--taskset")) {
-        taskset = 1;
+        cpuset_output_format = HWLOC_UTILS_CPUSET_FORMAT_TASKSET;
         goto next;
       }
       if (!strcmp (argv[0], "-e") || !strncmp (argv[0], "--get-last-cpu-location", 10)) {
@@ -217,73 +334,44 @@ int main(int argc, char *argv[])
 	  membind_policy = HWLOC_MEMBIND_BIND;
 	else if (!strncmp(argv[1], "interleave", 2))
 	  membind_policy = HWLOC_MEMBIND_INTERLEAVE;
+	else if (!strncmp(argv[1], "weighted", 2))
+	  membind_policy = HWLOC_MEMBIND_WEIGHTED_INTERLEAVE;
 	else if (!strncmp(argv[1], "nexttouch", 2))
 	  membind_policy = HWLOC_MEMBIND_NEXTTOUCH;
 	else {
 	  fprintf(stderr, "Unrecognized memory binding policy %s\n", argv[1]);
-          usage ("hwloc-bind", stderr);
+          usage (callname, stderr);
           exit(EXIT_FAILURE);
 	}
 	got_mempolicy = 1;
 	opt = 1;
 	goto next;
       }
-      if (!strcmp(argv[0], "--hbm")) {
-	only_hbm = 1;
-	goto next;
-      }
-      if (!strcmp(argv[0], "--no-hbm")) {
-	only_hbm = 0;
-	goto next;
-      }
-      if (!strcmp (argv[0], "--whole-system")) {
-	if (loaded) {
-	  fprintf(stderr, "Input option %s disallowed after options using the topology\n", argv[0]);
-	  exit(EXIT_FAILURE);
-	}
-	flags |= HWLOC_TOPOLOGY_FLAG_WHOLE_SYSTEM;
-	goto next;
-      }
-      if (!strcmp (argv[0], "--restrict")) {
-	hwloc_bitmap_t restrictset;
-	int err;
-	if (argc < 2) {
-	  usage (callname, stdout);
-	  exit(EXIT_FAILURE);
-	}
-	restrictset = hwloc_bitmap_alloc();
-	hwloc_bitmap_sscanf(restrictset, argv[1]);
-	ENSURE_LOADED();
-	err = hwloc_topology_restrict (topology, restrictset, 0);
-	if (err) {
-	  perror("Restricting the topology");
-	  /* FALLTHRU */
-	}
-	hwloc_bitmap_free(restrictset);
-	argc--;
-	argv++;
-	goto next;
-      }
 
       fprintf (stderr, "Unrecognized option: %s\n", argv[0]);
-      usage("hwloc-bind", stderr);
+      usage(callname, stderr);
       return EXIT_FAILURE;
     }
 
-    ENSURE_LOADED();
+    location = argv[0];
+    nodeset_location = 0;
+    if (!strncmp(location, "nodeset=", 8)) {
+      location += 8;
+      nodeset_location = 1;
+    }
 
     lcontext.topology = topology;
     lcontext.topodepth = depth;
     lcontext.only_hbm = only_hbm;
     lcontext.logical = logical;
     lcontext.verbose = verbose;
-    scontext.nodeset_input = use_nodeset;
-    scontext.nodeset_output = working_on_cpubind ? 0 : 1;
-    scontext.output_set = working_on_cpubind ? cpubind_set : membind_set;
-    ret = hwloc_calc_process_location_as_set(&lcontext, &scontext, argv[0]);
+    scontext.nodeset_input = use_nodeset || nodeset_location;
+    scontext.cpuset_input_format = HWLOC_UTILS_CPUSET_FORMAT_UNKNOWN;
+    scontext.output_cpuset = working_on_cpubind ? cpubind_set : dummy_set; /* don't modify cpubind_set if adding membind locations */
+    scontext.output_nodeset = working_on_cpubind ? dummy_set : membind_set; /* don't modify membind_set if adding cpubind locations */
+    ret = hwloc_calc_process_location_as_set(&lcontext, &scontext, location);
     if (ret < 0) {
-      if (verbose > 0)
-	fprintf(stderr, "assuming the command starts at %s\n", argv[0]);
+      fprintf(stderr, "argument `%s' unrecognized, assuming this is the executable.\n", argv[0]);
       break;
     }
     if (working_on_cpubind)
@@ -296,8 +384,6 @@ int main(int argc, char *argv[])
     argv += opt+1;
   }
 
-  ENSURE_LOADED();
-
   if (pid_number > 0 && tid_number > 0) {
     fprintf(stderr, "cannot operate both on tid and pid\n");
     return EXIT_FAILURE;
@@ -323,6 +409,11 @@ int main(int argc, char *argv[])
     return EXIT_FAILURE;
   }
 
+  if (!got_cpubind && !got_membind && !get_binding && !get_last_cpu_location) {
+    if (verbose >= 0)
+      fprintf(stderr, "got neither CPU nor memory binding locations.\n");
+  }
+
   if (get_binding || get_last_cpu_location) {
     char *s;
     const char *policystr = NULL;
@@ -360,16 +451,10 @@ int main(int argc, char *argv[])
       if (use_nodeset) {
 	hwloc_bitmap_t nset = hwloc_bitmap_alloc();
 	hwloc_cpuset_to_nodeset(topology, cpubind_set, nset);
-	if (taskset)
-	  hwloc_bitmap_taskset_asprintf(&s, nset);
-	else
-	  hwloc_bitmap_asprintf(&s, nset);
+        hwloc_utils_cpuset_format_asprintf(&s, nset, cpuset_output_format);
 	hwloc_bitmap_free(nset);
       } else {
-	if (taskset)
-	  hwloc_bitmap_taskset_asprintf(&s, cpubind_set);
-	else
-	  hwloc_bitmap_asprintf(&s, cpubind_set);
+        hwloc_utils_cpuset_format_asprintf(&s, cpubind_set, cpuset_output_format);
       }
 
       } else {
@@ -389,14 +474,12 @@ int main(int argc, char *argv[])
 	  fprintf(stderr, "hwloc_get_membind failed (errno %d %s)\n", errno, errmsg);
 	return EXIT_FAILURE;
       }
-      if (taskset)
-	hwloc_bitmap_taskset_asprintf(&s, membind_set);
-      else
-	hwloc_bitmap_asprintf(&s, membind_set);
+      hwloc_utils_cpuset_format_asprintf(&s, membind_set, cpuset_output_format);
       switch (policy) {
       case HWLOC_MEMBIND_FIRSTTOUCH: policystr = "firsttouch"; break;
       case HWLOC_MEMBIND_BIND: policystr = "bind"; break;
       case HWLOC_MEMBIND_INTERLEAVE: policystr = "interleave"; break;
+      case HWLOC_MEMBIND_WEIGHTED_INTERLEAVE: policystr = "weighted interleave"; break;
       case HWLOC_MEMBIND_NEXTTOUCH: policystr = "nexttouch"; break;
       default: fprintf(stderr, "unknown memory policy %d\n", policy); assert(0); break;
       }
@@ -415,6 +498,78 @@ int main(int argc, char *argv[])
       if (!force)
 	goto failed_binding;
     }
+
+    if (best_memattr_str && !hwloc_bitmap_iszero(membind_set)) {
+      unsigned nrnodes = hwloc_bitmap_weight(membind_set);
+      hwloc_obj_t *nodes;
+      unsigned i;
+      int j;
+      unsigned long best_node_flags;
+      struct hwloc_location loc;
+      char *s;
+
+      best_node_flags = hwloc_utils_parse_best_node_flags(best_memattr_str);
+
+      best_memattr_id = hwloc_utils_parse_memattr_name(topology, best_memattr_str);
+      if (best_memattr_id == (hwloc_memattr_id_t) -1) {
+        fprintf(stderr, "unrecognized memattr %s\n", best_memattr_str);
+        return EXIT_FAILURE;
+      }
+
+      loc.type = HWLOC_LOCATION_TYPE_CPUSET;
+      loc.location.cpuset = cpubind_set;
+      if (verbose > 0) {
+        hwloc_bitmap_asprintf(&s, membind_set);
+        fprintf(stderr, "memory binding set was %s before filtering by best memattr\n", s);
+        free(s);
+      }
+
+      nodes = malloc(nrnodes *sizeof(*nodes));
+      if (!nodes) {
+        fprintf(stderr, "failed to allocate nodes array for finding best node(s).\n");
+        return EXIT_FAILURE;
+      }
+
+      for(i=0, j=hwloc_bitmap_first(membind_set);
+          i<nrnodes;
+          i++, j=hwloc_bitmap_next(membind_set, j))
+        nodes[i] = hwloc_get_numanode_obj_by_os_index(topology, j);
+
+      ret = hwloc_utils_get_best_node_in_array_by_memattr(topology, best_memattr_id, nrnodes, nodes, &loc, best_node_flags, membind_set, verbose);
+
+      free(nodes);
+
+      if (ret < 0 || hwloc_bitmap_iszero(membind_set)) {
+        fprintf(stderr, "failed to find best memory node(s) for memory attribute `%s' among the given membind set.\n", best_memattr_str);
+        return EXIT_FAILURE;
+      }
+
+      if (verbose > 0) {
+        hwloc_bitmap_asprintf(&s, membind_set);
+        /* double-space before %s for alignment with previous verbose message */
+        fprintf(stderr, "memory binding is now  %s after filtering by best memattr\n", s);
+        free(s);
+      }
+    }
+
+    if (default_nodes && !hwloc_bitmap_iszero(membind_set)) {
+      hwloc_bitmap_t default_nodeset = hwloc_bitmap_alloc();
+      if (!default_nodeset) {
+        fprintf(stderr, "failed to allocate default nodeset.\n");
+        return EXIT_FAILURE;
+      }
+
+      ret = hwloc_topology_get_default_nodeset(topology, default_nodeset, 0);
+      if (ret < 0) {
+        fprintf(stderr, "failed to get default nodeset.\n");
+        return EXIT_FAILURE;
+      }
+
+      hwloc_bitmap_and(membind_set, membind_set, default_nodeset);
+
+      hwloc_bitmap_free(default_nodeset);
+    }
+
     if (verbose > 0) {
       char *s;
       hwloc_bitmap_asprintf(&s, membind_set);
@@ -435,10 +590,10 @@ int main(int argc, char *argv[])
       char *s;
       hwloc_bitmap_asprintf(&s, membind_set);
       if (pid_number > 0)
-        fprintf(stderr, "hwloc_set_proc_membind %s (policy %d flags %x) PID %d failed (errno %d %s)\n",
+        fprintf(stderr, "hwloc_set_proc_membind %s (policy %d flags 0x%x) PID %d failed (errno %d %s)\n",
 		s, membind_policy, membind_flags, pid_number, bind_errno, errmsg);
       else
-        fprintf(stderr, "hwloc_set_membind %s (policy %d flags %x) failed (errno %d %s)\n",
+        fprintf(stderr, "hwloc_set_membind %s (policy %d flags 0x%x) failed (errno %d %s)\n",
 		s, membind_policy, membind_flags, bind_errno, errmsg);
       free(s);
     }
@@ -449,6 +604,11 @@ int main(int argc, char *argv[])
       fprintf(stderr, "--mempolicy ignored unless memory binding is also requested with --membind.\n");
   }
 
+  if (!got_cpubind && no_smt != -1) {
+    hwloc_bitmap_copy(cpubind_set, hwloc_topology_get_topology_cpuset(topology));
+    got_cpubind = 1;
+  }
+
   if (got_cpubind) {
     if (hwloc_bitmap_iszero(cpubind_set)) {
       if (verbose >= 0)
@@ -467,6 +627,13 @@ int main(int argc, char *argv[])
 	fprintf(stderr, "Conflicting CPU and memory binding requested, adding HWLOC_CPUBIND_NOMEMBIND flag.\n");
       cpubind_flags |= HWLOC_CPUBIND_NOMEMBIND;
     }
+    if (no_smt != -1) {
+      if (hwloc_get_type_depth(topology, HWLOC_OBJ_CORE) == HWLOC_TYPE_DEPTH_UNKNOWN) {
+	fprintf(stderr, "Topology has no Core object, ignoring --no-smt\n");
+      } else {
+	hwloc_bitmap_singlify_per_core(topology, cpubind_set, no_smt);
+      }
+    }
     if (single)
       hwloc_bitmap_singlify(cpubind_set);
     if (pid_number > 0)
@@ -483,13 +650,13 @@ int main(int argc, char *argv[])
       char *s;
       hwloc_bitmap_asprintf(&s, cpubind_set);
       if (pid_number > 0)
-        fprintf(stderr, "hwloc_set_proc_cpubind %s (flags %x) PID %d failed (errno %d %s)\n",
+        fprintf(stderr, "hwloc_set_proc_cpubind %s (flags 0x%x) PID %d failed (errno %d %s)\n",
 		s, cpubind_flags, pid_number, bind_errno, errmsg);
       else if (tid_number > 0)
-        fprintf(stderr, "hwloc_set_tid_cpubind %s (flags %x) PID %d failed (errno %d %s)\n",
+        fprintf(stderr, "hwloc_set_tid_cpubind %s (flags 0x%x) PID %d failed (errno %d %s)\n",
 		s, cpubind_flags, tid_number, bind_errno, errmsg);
       else
-        fprintf(stderr, "hwloc_set_cpubind %s (flags %x) failed (errno %d %s)\n",
+        fprintf(stderr, "hwloc_set_cpubind %s (flags 0x%x) failed (errno %d %s)\n",
 		s, cpubind_flags, bind_errno, errmsg);
       free(s);
     }
@@ -499,6 +666,7 @@ int main(int argc, char *argv[])
 
   hwloc_bitmap_free(cpubind_set);
   hwloc_bitmap_free(membind_set);
+  hwloc_bitmap_free(dummy_set);
 
   hwloc_topology_destroy(topology);
 
@@ -528,6 +696,7 @@ int main(int argc, char *argv[])
 failed_binding:
   hwloc_bitmap_free(cpubind_set);
   hwloc_bitmap_free(membind_set);
+  hwloc_bitmap_free(dummy_set);
   hwloc_topology_destroy(topology);
   return EXIT_FAILURE;
 }
