@@ -85,7 +85,7 @@ static struct hwloc_calc_level intersect;
 static int hiernblevels;
 static struct hwloc_calc_level *hierlevels;
 static int local_numanodes = 0;
-static unsigned long local_numanode_flags = HWLOC_LOCAL_NUMANODE_FLAG_SMALLER_LOCALITY | HWLOC_LOCAL_NUMANODE_FLAG_LARGER_LOCALITY;
+static unsigned long local_numanode_flags = HWLOC_LOCAL_NUMANODE_FLAG_SMALLER_LOCALITY | HWLOC_LOCAL_NUMANODE_FLAG_LARGER_LOCALITY | HWLOC_LOCAL_NUMANODE_FLAG_INTERSECT_LOCALITY;
 static hwloc_memattr_id_t best_memattr_id = (hwloc_memattr_id_t) -1;
 static unsigned long best_node_flags = 0;
 static int showlargestobjs = 0;
@@ -327,7 +327,7 @@ hwloc_calc_output(hwloc_topology_t topology, const char *sep, hwloc_bitmap_t cpu
       if (!err) {
         unsigned i, first = 1;
         if (best_memattr_id != (hwloc_memattr_id_t) -1) {
-          err = hwloc_utils_get_best_node_in_array_by_memattr(topology, best_memattr_id, nrnodes, nodes, &loc, best_node_flags, nodeset_filter);
+          err = hwloc_utils_get_best_node_in_array_by_memattr(topology, best_memattr_id, nrnodes, nodes, &loc, best_node_flags, nodeset_filter, verbose);
           /* on error, nodeset_filter is zeroed, and we report nothing below (except if default flag is set) */
         }
         if (!sep)
