@@ -1,8 +1,9 @@
 /*
- * Copyright © 2009 CNRS
- * Copyright © 2009-2020 Inria.  All rights reserved.
+ * Copyright © 2009, 2024 CNRS
+ * Copyright © 2009-2025 Inria.  All rights reserved.
  * Copyright © 2009-2012 Université Bordeaux
  * Copyright © 2009-2011 Cisco Systems, Inc.  All rights reserved.
+ * Copyright © 2023 Université de Reims Champagne-Ardenne.  All rights reserved.
  * See COPYING in top-level directory.
  */
 
@@ -23,6 +24,10 @@
 #ifdef HAVE_STDINT_H
 #include <stdint.h>
 #endif
+#ifdef HAVE_DIRENT_H
+#include <dirent.h>
+#endif
+#include <fcntl.h>
 #include <assert.h>
 
 extern void usage(const char *name, FILE *where);
@@ -77,14 +82,19 @@ hwloc_utils_input_format_usage(FILE *where, int addspaces)
 	   addspaces, " ");
 }
 
-enum hwloc_utils_input_format {
-  HWLOC_UTILS_INPUT_DEFAULT,
-  HWLOC_UTILS_INPUT_XML,
-  HWLOC_UTILS_INPUT_FSROOT,
-  HWLOC_UTILS_INPUT_SYNTHETIC,
-  HWLOC_UTILS_INPUT_CPUID,
-  HWLOC_UTILS_INPUT_SHMEM
+struct hwloc_utils_input_format_s {
+  enum hwloc_utils_input_format {
+    HWLOC_UTILS_INPUT_DEFAULT,
+    HWLOC_UTILS_INPUT_XML,
+    HWLOC_UTILS_INPUT_FSROOT,
+    HWLOC_UTILS_INPUT_SYNTHETIC,
+    HWLOC_UTILS_INPUT_CPUID,
+    HWLOC_UTILS_INPUT_SHMEM,
+    HWLOC_UTILS_INPUT_ARCHIVE
+  } format;
+  int oldworkdir;
 };
+#define HWLOC_UTILS_INPUT_FORMAT_DEFAULT (struct hwloc_utils_input_format_s) { HWLOC_UTILS_INPUT_DEFAULT, -1 }
 
 static __hwloc_inline enum hwloc_utils_input_format
 hwloc_utils_parse_input_format(const char *name, const char *callname)
@@ -101,6 +111,8 @@ hwloc_utils_parse_input_format(const char *name, const char *callname)
     return HWLOC_UTILS_INPUT_SYNTHETIC;
   else if (!hwloc_strncasecmp(name, "cpuid", 1))
     return HWLOC_UTILS_INPUT_CPUID;
+  else if (!hwloc_strncasecmp(name, "archive", 1))
+    return HWLOC_UTILS_INPUT_ARCHIVE;
 
   fprintf(stderr, "input format `%s' not supported\n", name);
   usage(callname, stderr);
@@ -109,7 +121,7 @@ hwloc_utils_parse_input_format(const char *name, const char *callname)
 
 static __hwloc_inline int
 hwloc_utils_lookup_input_option(char *argv[], int argc, int *consumed_opts,
-				char **inputp, enum hwloc_utils_input_format *input_formatp,
+				char **inputp, struct hwloc_utils_input_format_s *input_formatp,
 				const char *callname)
 {
   if (!strcmp (argv[0], "--input")
@@ -131,7 +143,8 @@ hwloc_utils_lookup_input_option(char *argv[], int argc, int *consumed_opts,
       usage (callname, stderr);
       exit(EXIT_FAILURE);
     }
-    *input_formatp = hwloc_utils_parse_input_format (argv[1], callname);
+    *input_formatp = HWLOC_UTILS_INPUT_FORMAT_DEFAULT;
+    input_formatp->format = hwloc_utils_parse_input_format (argv[1], callname);
     *consumed_opts = 1;
     return 1;
   }
@@ -143,7 +156,8 @@ hwloc_utils_lookup_input_option(char *argv[], int argc, int *consumed_opts,
       exit(EXIT_FAILURE);
     }
     *inputp = argv[1];
-    *input_formatp = HWLOC_UTILS_INPUT_SYNTHETIC;
+    *input_formatp = HWLOC_UTILS_INPUT_FORMAT_DEFAULT;
+    input_formatp->format = HWLOC_UTILS_INPUT_SYNTHETIC;
     *consumed_opts = 1;
     return 1;
   } else if (!strcmp (argv[0], "--xml")) {
@@ -152,7 +166,8 @@ hwloc_utils_lookup_input_option(char *argv[], int argc, int *consumed_opts,
       exit(EXIT_FAILURE);
     }
     *inputp = argv[1];
-    *input_formatp = HWLOC_UTILS_INPUT_XML;
+    *input_formatp = HWLOC_UTILS_INPUT_FORMAT_DEFAULT;
+    input_formatp->format = HWLOC_UTILS_INPUT_XML;
     *consumed_opts = 1;
     return 1;
   } else if (!strcmp (argv[0], "--fsys-root")) {
@@ -161,7 +176,8 @@ hwloc_utils_lookup_input_option(char *argv[], int argc, int *consumed_opts,
       exit(EXIT_FAILURE);
     }
     *inputp = argv[1];
-    *input_formatp = HWLOC_UTILS_INPUT_FSROOT;
+    *input_formatp = HWLOC_UTILS_INPUT_FORMAT_DEFAULT;
+    input_formatp->format = HWLOC_UTILS_INPUT_FSROOT;
     *consumed_opts = 1;
     return 1;
   }
@@ -187,6 +203,12 @@ hwloc_utils_autodetect_input_format(const char *input, int verbose)
 	printf("assuming `%s' is a shmem topology file\n", input);
       return HWLOC_UTILS_INPUT_SHMEM;
     }
+    if ((len >= 7 && !strcmp(input+len-7, ".tar.gz"))
+        || (len >= 8 && !strcmp(input+len-8, ".tar.bz2"))) {
+      if (verbose > 0)
+	printf("assuming `%s' is an archive topology file\n", input);
+      return HWLOC_UTILS_INPUT_ARCHIVE;
+    }
     if (verbose > 0)
       printf("assuming `%s' is a XML file\n", input);
     return HWLOC_UTILS_INPUT_XML;
@@ -220,9 +242,10 @@ hwloc_utils_autodetect_input_format(const char *input, int verbose)
 static __hwloc_inline int
 hwloc_utils_enable_input_format(struct hwloc_topology *topology, unsigned long flags,
 				const char *input,
-				enum hwloc_utils_input_format *input_format,
+				struct hwloc_utils_input_format_s *input_formatp,
 				int verbose, const char *callname)
 {
+  enum hwloc_utils_input_format *input_format = &input_formatp->format;
   if (*input_format == HWLOC_UTILS_INPUT_DEFAULT && !strcmp(input, "-.xml")) {
     *input_format = HWLOC_UTILS_INPUT_XML;
     input = "-";
@@ -294,6 +317,86 @@ hwloc_utils_enable_input_format(struct hwloc_topology *topology, unsigned long f
     break;
   }
 
+  case HWLOC_UTILS_INPUT_ARCHIVE: {
+#ifdef HWLOC_LINUX_SYS
+    char mntpath[] = "/tmp/tmpdir.hwloc.archivemount.XXXXXX";
+    char mntcmd[512];
+    char umntcmd[512];
+    DIR *dir;
+    struct dirent *dirent;
+    /* oldworkdir == -1 -> close would fail if !defined(O_PATH), but we don't care */
+    struct hwloc_utils_input_format_s sub_input_format = HWLOC_UTILS_INPUT_FORMAT_DEFAULT;
+    char *subdir = NULL;
+    int err;
+
+#ifdef O_PATH
+    if (-1 == input_formatp->oldworkdir) { /* if archivemount'ed recursively, only keep the first oldworkdir */
+        sub_input_format.oldworkdir = open(".", O_DIRECTORY|O_PATH); /* more efficient than getcwd(3) */
+        if (sub_input_format.oldworkdir < 0) {
+            perror("Saving current working directory");
+            return EXIT_FAILURE;
+        }
+    }
+#endif
+    if (!mkdtemp(mntpath)) {
+      perror("Creating archivemount directory");
+      close(sub_input_format.oldworkdir);
+      return EXIT_FAILURE;
+    }
+    snprintf(mntcmd, sizeof(mntcmd), "archivemount -o ro %s %s", input, mntpath);
+    err = system(mntcmd);
+    if (err) {
+      perror("Archivemount'ing the archive");
+      rmdir(mntpath);
+      close(sub_input_format.oldworkdir);
+      return EXIT_FAILURE;
+    }
+    snprintf(umntcmd, sizeof(umntcmd), "umount -l %s", mntpath);
+
+    /* enter the mount point and stay there so that we can umount+rmdir immediately but still use it later */
+    if (chdir(mntpath) < 0) {
+      perror("Entering the archivemount'ed archive");
+      if (system(umntcmd) < 0)
+        perror("Unmounting the archivemount'ed archive (ignored)");
+      rmdir(mntpath);
+      close(sub_input_format.oldworkdir);
+      return EXIT_FAILURE;
+    }
+    if (system(umntcmd) < 0)
+      perror("Unmounting the archivemount'ed archive (ignored)");
+    rmdir(mntpath);
+
+    /* there should be a single subdirectory in the archive */
+    dir = opendir(".");
+    while ((dirent = readdir(dir)) != NULL) {
+      if (strcmp(dirent->d_name, ".") && strcmp(dirent->d_name, "..")) {
+        subdir = dirent->d_name;
+        break;
+      }
+    }
+    closedir(dir);
+
+    if (!subdir) {
+      perror("No subdirectory in archivemount directory");
+      close(sub_input_format.oldworkdir);
+      return EXIT_FAILURE;
+    }
+
+    /* call ourself recursively on subdir, it should be either a fsroot or a cpuid directory */
+    err = hwloc_utils_enable_input_format(topology, flags, subdir, &sub_input_format, verbose, callname);
+    if (!err)
+      *input_formatp = sub_input_format;
+    else {
+      close(sub_input_format.oldworkdir);
+      return err;
+    }
+#else
+    fprintf(stderr, "This installation of hwloc does not support loading from an archive, sorry.\n");
+    exit(EXIT_FAILURE);
+#endif
+    break;
+  }
+
   case HWLOC_UTILS_INPUT_SYNTHETIC:
     if (hwloc_topology_set_synthetic(topology, input)) {
       perror("Setting synthetic topology description");
@@ -311,37 +414,104 @@ hwloc_utils_enable_input_format(struct hwloc_topology *topology, unsigned long f
   return 0;
 }
 
+static __hwloc_inline void
+hwloc_utils_disable_input_format(struct hwloc_utils_input_format_s *input_format)
+{
+  if (-1 < input_format->oldworkdir) {
+#ifdef HWLOC_LINUX_SYS
+#ifdef O_PATH
+    int err = fchdir(input_format->oldworkdir);
+    if (err) {
+      perror("Restoring current working directory");
+    }
+#else
+    fprintf(stderr, "Couldn't restore working directory. Errors may arise.\n");
+#endif
+    close(input_format->oldworkdir);
+    input_format->oldworkdir = -1;
+#else
+    fprintf(stderr, "oldworkdir should not have been changed. You should not have reached this execution branch.\n");
+    exit(EXIT_FAILURE);
+#endif
+  }
+}
+
 static __hwloc_inline void
 hwloc_utils_print_distance_matrix(FILE *output, unsigned nbobjs, hwloc_obj_t *objs, hwloc_uint64_t *matrix, int logical, int show_types)
 {
   unsigned i, j;
+#define MATRIX_ITEM_SIZE_MAX 17 /* 16 + ending \0 */
+  char *headers;
+  char *values;
+  char *buf;
+  size_t len, max;
 
-  /* column header */
-  fprintf(output, "  index");
-  for(j=0; j<nbobjs; j++) {
-    if (show_types)
-      fprintf(output, " %s:%d",
-	      hwloc_obj_type_string(objs[j]->type),
-	      (int) (logical ? objs[j]->logical_index : objs[j]->os_index));
-    else
-      fprintf(output, " % 5d",
-	      (int) (logical ? objs[j]->logical_index : objs[j]->os_index));
+  headers = malloc((nbobjs+1)*MATRIX_ITEM_SIZE_MAX);
+  values = malloc(nbobjs*nbobjs*MATRIX_ITEM_SIZE_MAX);
+  if (!headers || !values) {
+    free(headers);
+    free(values);
+    return;
   }
-  fprintf(output, "\n");
 
-  /* each line */
-  for(i=0; i<nbobjs; i++) {
-    /* row header */
-    fprintf(output, "  % 5d",
-	    (int) (logical ? objs[i]->logical_index : objs[i]->os_index));
-
-    /* row values */
-    for(j=0; j<nbobjs; j++) {
-      for(j=0; j<nbobjs; j++)
-	fprintf(output, " % 5d", (int) matrix[i*nbobjs+j]);
-      fprintf(output, "\n");
+  snprintf(headers, MATRIX_ITEM_SIZE_MAX, "           index" /* 16 */);
+  max = 5;
+  /* prepare column headers */
+  for(i=0, buf = headers + MATRIX_ITEM_SIZE_MAX;
+      i<nbobjs;
+      i++, buf += MATRIX_ITEM_SIZE_MAX) {
+    char tmp[MATRIX_ITEM_SIZE_MAX];
+    hwloc_obj_t obj = objs[i];
+    unsigned index = logical ? obj->logical_index : obj->os_index;
+    if (obj->type == HWLOC_OBJ_OS_DEVICE)
+      len = snprintf(tmp, MATRIX_ITEM_SIZE_MAX,
+                     "%s", obj->name);
+    else if (obj->type == HWLOC_OBJ_PCI_DEVICE)
+      len = snprintf(tmp, MATRIX_ITEM_SIZE_MAX,
+                     "%04x:%02x:%02x.%01x",
+                     obj->attr->pcidev.domain, obj->attr->pcidev.bus, obj->attr->pcidev.dev, obj->attr->pcidev.func);
+    else if (show_types)
+      len = snprintf(tmp, MATRIX_ITEM_SIZE_MAX,
+                     "%s:%d", hwloc_obj_type_string(obj->type), (int) index);
+    else
+      len = snprintf(tmp, MATRIX_ITEM_SIZE_MAX,
+                     "%d", (int) index);
+    if (len >= max)
+      max = len;
+    /* store it at the end of the slot in headers */
+    memcpy(buf + (MATRIX_ITEM_SIZE_MAX - len - 1), tmp, len+1);
+    /* and pad with spaces at the begining */
+    memset(buf, ' ', MATRIX_ITEM_SIZE_MAX - len - 1);
+  }
+  /* prepare values */
+  for(i=0, buf = values;
+      i<nbobjs;
+      i++) {
+    for(j=0; j<nbobjs; j++, buf += MATRIX_ITEM_SIZE_MAX) {
+     char tmp[MATRIX_ITEM_SIZE_MAX];
+     len = snprintf(tmp, MATRIX_ITEM_SIZE_MAX, "%llu", (unsigned long long) matrix[i*nbobjs+j]);
+      if (len >= max)
+        max = len;
+      /* store it at the end of the slot in values */
+      memcpy(buf + (MATRIX_ITEM_SIZE_MAX - len - 1), tmp, len+1);
+      /* and pad with spaces at the begining */
+      memset(buf, ' ', MATRIX_ITEM_SIZE_MAX - len - 1);
     }
   }
+
+  /* now display everything */
+  for(i=0; i<nbobjs + 1; i++)
+    fprintf(output, " %s", headers + i*MATRIX_ITEM_SIZE_MAX + MATRIX_ITEM_SIZE_MAX-max-1);
+  fprintf(output, "\n");
+  for(i=0; i<nbobjs; i++) {
+    fprintf(output, " %s", headers + (i+1)*MATRIX_ITEM_SIZE_MAX + MATRIX_ITEM_SIZE_MAX-max-1);
+    for(j=0; j<nbobjs; j++)
+      fprintf(output, " %s", values + (i*nbobjs+j)*MATRIX_ITEM_SIZE_MAX + MATRIX_ITEM_SIZE_MAX-max-1);
+    fprintf(output, "\n");
+  }
+
+  free(headers);
+  free(values);
 }
 
 static __hwloc_inline int
@@ -530,17 +700,15 @@ hwloc_utils_parse_flags(char * str, struct hwloc_utils_parsing_flag possible_fla
   char *end;
   int ul_flag;
   int i;
-  int j = 0;
+  size_t j;
   unsigned long ul_flags = 0;
 
   ul_flag = strtoul(str, &end, 0);
   if(end != str && *end == '\0')
     return ul_flag;
 
-  while (str[j]) {
+  for(j=0; str[j]; j++)
     str[j] = toupper(str[j]);
-    j++;
-  }
 
   if(strcmp(str, "NONE") == 0)
     return 0;
@@ -635,143 +803,314 @@ hwloc_utils_parse_memattr_name(hwloc_topology_t topo, const char *str)
     return id;
 }
 
+#define HWLOC_UTILS_BEST_NODE_FLAG_DEFAULT (1UL<<0) /* report all nodes if no best found */
+#define HWLOC_UTILS_BEST_NODE_FLAG_STRICT (1UL<<1) /* report only best with same initiator */
+
+static __hwloc_inline unsigned long
+hwloc_utils_parse_best_node_flags(char *str)
+{
+  unsigned long best_memattr_flags = 0;
+  char *tmp;
+
+  tmp = strstr(str, ",default");
+  if (tmp) {
+    memmove(tmp, tmp+8, strlen(tmp+8)+1);
+    best_memattr_flags |= HWLOC_UTILS_BEST_NODE_FLAG_DEFAULT;
+  }
+
+  tmp = strstr(str, ",strict");
+  if (tmp) {
+    memmove(tmp, tmp+7, strlen(tmp+7)+1);
+    best_memattr_flags |= HWLOC_UTILS_BEST_NODE_FLAG_STRICT;
+  }
+
+  return best_memattr_flags;
+}
+
+static __hwloc_inline void
+hwloc_utils__update_best_node(hwloc_obj_t newnode, uint64_t newvalue,
+                              uint64_t *bestvalue, hwloc_bitmap_t bestnodeset,
+                              unsigned long mflags)
+{
+  if (hwloc_bitmap_iszero(bestnodeset)) {
+    /* first */
+    *bestvalue = newvalue;
+    hwloc_bitmap_only(bestnodeset, newnode->os_index);
+
+  } else if (mflags & HWLOC_MEMATTR_FLAG_HIGHER_FIRST) {
+    if (newvalue > *bestvalue) {
+      /* higher */
+      *bestvalue = newvalue;
+      hwloc_bitmap_only(bestnodeset, newnode->os_index);
+    } else if (newvalue == *bestvalue) {
+      /* as high */
+      hwloc_bitmap_set(bestnodeset, newnode->os_index);
+    }
+
+  } else {
+    assert(mflags & HWLOC_MEMATTR_FLAG_LOWER_FIRST);
+    if (newvalue < *bestvalue) {
+      /* lower */
+      *bestvalue = newvalue;
+      hwloc_bitmap_only(bestnodeset, newnode->os_index);
+    } else if (newvalue == *bestvalue) {
+      /* as low */
+      hwloc_bitmap_set(bestnodeset, newnode->os_index);
+    }
+  }
+}
+
+/* fill best_nodeset with best nodes.
+ * if STRICT flag, only the really local ones are returned.
+ * if none is best (they don't have values), return empty.
+ * if none is best and DEFAULT flag, return all nodes.
+ * on error, return empty.
+ */
 static __hwloc_inline int
 hwloc_utils_get_best_node_in_array_by_memattr(hwloc_topology_t topology, hwloc_memattr_id_t id,
                                               unsigned nbnodes, hwloc_obj_t *nodes,
-                                              struct hwloc_location *initiator)
+                                              struct hwloc_location *initiator,
+                                              unsigned long flags,
+                                              hwloc_nodeset_t best_nodeset,
+                                              int verbose)
 {
-  unsigned nbtgs, i, j;
-  hwloc_obj_t *tgs;
-  int best;
-  hwloc_uint64_t *values, bestvalue;
+  unsigned i, j, nb = 0;
+  hwloc_uint64_t *values, bestvalue = 0;
   unsigned long mflags;
   int err;
 
+  hwloc_bitmap_zero(best_nodeset);
+
   err = hwloc_memattr_get_flags(topology, id, &mflags);
   if (err < 0)
     goto out;
 
-  nbtgs = 0;
-  err = hwloc_memattr_get_targets(topology, id, initiator, 0, &nbtgs, NULL, NULL);
-  if (err < 0)
-    goto out;
+  if (mflags & HWLOC_MEMATTR_FLAG_NEED_INITIATOR) {
+    /* iterate over targets, and then on their initiators */
+    for(i=0, nb = 0; i<nbnodes; i++) {
+      unsigned nbi;
+      struct hwloc_location *initiators;
 
-  tgs = malloc(nbtgs * sizeof(*tgs));
-  values = malloc(nbtgs * sizeof(*values));
-  if (!tgs || !values)
-    goto out_with_arrays;
-
-  err = hwloc_memattr_get_targets(topology, id, initiator, 0, &nbtgs, tgs, values);
-  if (err < 0)
-    goto out_with_arrays;
-
-  best = -1;
-  bestvalue = 0;
-  for(i=0; i<nbnodes; i++) {
-    for(j=0; j<nbtgs; j++)
-      if (tgs[j] == nodes[i])
-        break;
-    if (j==nbtgs)
-      /* no target info for this node */
-      continue;
-    if (best == -1) {
-      best = i;
-      bestvalue = values[j];
-    } else if (mflags & HWLOC_MEMATTR_FLAG_HIGHER_FIRST) {
-      if (values[j] > bestvalue) {
-        best = i;
-        bestvalue = values[j];
+      nbi = 0;
+      err = hwloc_memattr_get_initiators(topology, id, nodes[i], 0, &nbi, NULL, NULL);
+      if (err < 0) {
+        hwloc_bitmap_zero(best_nodeset);
+        goto none;
       }
-    } else {
-      assert(mflags & HWLOC_MEMATTR_FLAG_LOWER_FIRST);
-      if (values[j] < bestvalue) {
-        best = i;
-        bestvalue = values[j];
+
+      initiators = malloc(nbi * sizeof(*initiators));
+      values = malloc(nbi * sizeof(*values));
+      if (!initiators || !values) {
+        hwloc_bitmap_zero(best_nodeset);
+        free(initiators);
+        free(values);
+        goto none;
+      }
+
+      err = hwloc_memattr_get_initiators(topology, id, nodes[i], 0, &nbi, initiators, values);
+      if (err < 0) {
+        hwloc_bitmap_zero(best_nodeset);
+        free(initiators);
+        free(values);
+        goto none;
+      }
+
+      for(j=0; j<nbi; j++) {
+        /* does this initiator match the user location? */
+        if (initiator->type != initiators[j].type)
+          continue;
+        switch (initiator->type) {
+        case HWLOC_LOCATION_TYPE_OBJECT:
+          if (initiator->location.object->type != initiators[j].location.object->type
+              || initiator->location.object->gp_index != initiators[j].location.object->gp_index)
+            continue;
+          break;
+        case HWLOC_LOCATION_TYPE_CPUSET:
+          if (flags & HWLOC_UTILS_BEST_NODE_FLAG_STRICT) {
+            if (!hwloc_bitmap_isincluded(initiator->location.cpuset, initiators[j].location.cpuset))
+              continue;
+          } else {
+            if (!hwloc_bitmap_intersects(initiator->location.cpuset, initiators[j].location.cpuset))
+              continue;
+          }
+          break;
+        default:
+          abort();
+        }
+
+        hwloc_utils__update_best_node(nodes[i], values[j],
+                                      &bestvalue, best_nodeset,
+                                      mflags);
+        nb++;
+        break; /* there shouldn't be multiple initiators */
+      }
+
+      free(initiators);
+      free(values);
+    }
+
+  } else {
+    /* no initiator, just iterate over targets */
+    for(i=0; i<nbnodes; i++) {
+      uint64_t value;
+      if (!hwloc_memattr_get_value(topology, id, nodes[i], NULL, 0, &value)) {
+        hwloc_utils__update_best_node(nodes[i], value,
+                                      &bestvalue, best_nodeset,
+                                      mflags);
+        nb++;
       }
     }
   }
 
-  free(tgs);
-  free(values);
-  return best;
+  if (hwloc_bitmap_iszero(best_nodeset)) {
+  none:
+    if (flags & HWLOC_UTILS_BEST_NODE_FLAG_DEFAULT) {
+      if (!nb) {
+        if (verbose > 0)
+          printf("found no node with attribute values for this initiator, falling back to default\n");
+      } else {
+        fprintf(stderr, "couldn't find attribute values for all nodes, falling back to default\n");
+      }
+
+      /* try to get default nodes only */
+      hwloc_bitmap_t default_nodeset = hwloc_bitmap_alloc();
+      if (default_nodeset) {
+        err = hwloc_topology_get_default_nodeset(topology, default_nodeset, 0);
+        if (!err) {
+          hwloc_bitmap_zero(best_nodeset);
+          for(i=0; i<nbnodes; i++)
+            if (hwloc_bitmap_isset(default_nodeset, nodes[i]->os_index))
+              hwloc_bitmap_set(best_nodeset, nodes[i]->os_index);
+        }
+      }
+      free(default_nodeset);
+      /* if still empty, use all local nodes */
+      if (hwloc_bitmap_iszero(best_nodeset))
+        for(i=0; i<nbnodes; i++)
+          hwloc_bitmap_set(best_nodeset, nodes[i]->os_index);
+
+    } else {
+      if (!nb) {
+        if (verbose > 0)
+          printf("found no node with attribute values for this initiator\n");
+      } else {
+        fprintf(stderr, "couldn't find attribute values for all nodes, returning none\n");
+      }
+    }
+  }
+  return 0;
 
- out_with_arrays:
-  free(tgs);
-  free(values);
  out:
   return -1;
 }
 
-static __hwloc_inline int
-hwloc_utils_get_best_node_in_nodeset_by_memattr(hwloc_topology_t topology, hwloc_memattr_id_t id,
-                                                hwloc_nodeset_t nodeset,
-                                                struct hwloc_location *initiator)
+enum hwloc_utils_cpuset_format_e {
+  HWLOC_UTILS_CPUSET_FORMAT_UNKNOWN,
+  HWLOC_UTILS_CPUSET_FORMAT_HWLOC,
+  HWLOC_UTILS_CPUSET_FORMAT_LIST,
+  HWLOC_UTILS_CPUSET_FORMAT_SYSTEMD,
+  HWLOC_UTILS_CPUSET_FORMAT_TASKSET
+};
+
+static __hwloc_inline enum hwloc_utils_cpuset_format_e
+hwloc_utils_parse_cpuset_format(const char *string)
 {
-  unsigned nbtgs, i, j;
-  hwloc_obj_t *tgs;
-  int best;
-  hwloc_uint64_t *values, bestvalue;
-  unsigned long mflags;
-  int err;
-
-  err = hwloc_memattr_get_flags(topology, id, &mflags);
-  if (err < 0)
-    goto out;
-
-  nbtgs = 0;
-  err = hwloc_memattr_get_targets(topology, id, initiator, 0, &nbtgs, NULL, NULL);
-  if (err < 0)
-    goto out;
-
-  tgs = malloc(nbtgs * sizeof(*tgs));
-  values = malloc(nbtgs * sizeof(*values));
-  if (!tgs || !values)
-    goto out_with_arrays;
-
-  err = hwloc_memattr_get_targets(topology, id, initiator, 0, &nbtgs, tgs, values);
-  if (err < 0)
-    goto out_with_arrays;
-
-  best = -1;
-  bestvalue = 0;
-  hwloc_bitmap_foreach_begin(i, nodeset) {
-    for(j=0; j<nbtgs; j++)
-      if (tgs[j]->os_index == i)
-        break;
-    if (j==nbtgs)
-      /* no target info for this node */
-      continue;
-    if (best == -1) {
-      best = i;
-      bestvalue = values[j];
-    } else if (mflags & HWLOC_MEMATTR_FLAG_HIGHER_FIRST) {
-      if (values[j] > bestvalue) {
-        best = i;
-        bestvalue = values[j];
-      }
-    } else {
-      assert(mflags & HWLOC_MEMATTR_FLAG_LOWER_FIRST);
-      if (values[j] < bestvalue) {
-        best = i;
-        bestvalue = values[j];
-      }
-    }
-  } hwloc_bitmap_foreach_end();
-
-  if (best == -1)
-    hwloc_bitmap_zero(nodeset);
+  if (!strcmp(string, "hwloc"))
+    return HWLOC_UTILS_CPUSET_FORMAT_HWLOC;
+  else if (!strcmp(string, "list"))
+    return HWLOC_UTILS_CPUSET_FORMAT_LIST;
+  else if (!strcmp(string, "systemd-dbus-api"))
+    return HWLOC_UTILS_CPUSET_FORMAT_SYSTEMD;
+  else if (!strcmp(string, "taskset"))
+    return HWLOC_UTILS_CPUSET_FORMAT_TASKSET;
   else
-    hwloc_bitmap_only(nodeset, best);
+    return HWLOC_UTILS_CPUSET_FORMAT_UNKNOWN;
+}
 
-  free(tgs);
-  free(values);
-  return 0;
+/* The AllowedCPUs and AllowedMemoryNodes systemd DBus API syntax expects a string as follows:
+ * "ay 0xNNNN 0xAA [0xBB [...]]"
+ * where:
+ *   "0xNNNN" is the size of the array, max size 2^26.
+ *   "0xAA [0xBB [...]]" is the cpuset or nodeset mask given bytes after bytes, little endian order.
+ * e.g. the output of `hwloc-calc --cpuset-input-format list 0,31-32,63-64,77 --cpuset-output-format systemd-dbus-api` is
+ *   "ay 0x000a 0x01 0x00 0x00 x80 0x01 0x00 0x00 0x80 0x01 0x20
+ * e.g. the output of `hwloc-calc node:0 node:1 --nodeset-output-format systemd-dbus-api` is
+ *   "ay 0x0001 0x03"
+ * */
+#define HWLOC_SYSTEMD_DBUS_API_PREFIX_FORMAT "ay 0x%04x"
+#define HWLOC_SYSTEMD_DBUS_API_PREFIX_SIZE 9
+#define HWLOC_SYSTEMD_DBUS_API_BYTES_FORMAT " 0x%02x"
+#define HWLOC_SYSTEMD_DBUS_API_BYTES_SIZE 5
+static __hwloc_inline int hwloc_utils_systemd_asprintf(char ** strp, const struct hwloc_bitmap_s * __hwloc_restrict set)
+{
+  int last = hwloc_bitmap_last(set);
+  if (last == -1 ) {
+    fprintf(stderr, "Empty and infinite sets are not supported with the systemd-dbus-api output format\n");
+    exit(EXIT_FAILURE);
+  }
+  int bytes = last / 8 + 1; /* how many bytes to represent the bit mask */
+  int buflen = HWLOC_SYSTEMD_DBUS_API_PREFIX_SIZE + HWLOC_SYSTEMD_DBUS_API_BYTES_SIZE * bytes + 1;
+  int ret = 0;
+  char *buf;
+  buf = malloc(buflen);
+  *strp = buf;
+  ret = snprintf(buf, buflen, HWLOC_SYSTEMD_DBUS_API_PREFIX_FORMAT, bytes);
+  unsigned long ith = 0;
+  for (int byte=0; byte < bytes; byte++) {
+    if (byte % HWLOC_SIZEOF_UNSIGNED_LONG == 0) {
+      ith = hwloc_bitmap_to_ith_ulong(set, byte / HWLOC_SIZEOF_UNSIGNED_LONG);
+    }
+    ret += snprintf(buf + ret, HWLOC_SYSTEMD_DBUS_API_BYTES_SIZE + 1, HWLOC_SYSTEMD_DBUS_API_BYTES_FORMAT, (unsigned char) (ith & 0xFF));
+    ith >>= 8;
+  }
+  assert(ith == 0); /* whenever some bytes of the ulong ith may not be consumed, they must be 0 */
+  return ret;
+}
 
- out_with_arrays:
-  free(tgs);
-  free(values);
- out:
-  return -1;
+static __hwloc_inline int
+hwloc_utils_cpuset_format_asprintf(char **string, hwloc_const_bitmap_t set,
+                                   enum hwloc_utils_cpuset_format_e cpuset_output_format)
+{
+  switch (cpuset_output_format) {
+  case HWLOC_UTILS_CPUSET_FORMAT_HWLOC: return hwloc_bitmap_asprintf(string, set);
+  case HWLOC_UTILS_CPUSET_FORMAT_LIST: return hwloc_bitmap_list_asprintf(string, set);
+  case HWLOC_UTILS_CPUSET_FORMAT_SYSTEMD: return hwloc_utils_systemd_asprintf(string, set);
+  case HWLOC_UTILS_CPUSET_FORMAT_TASKSET: return hwloc_bitmap_taskset_asprintf(string, set);
+  default: abort();
+  }
+}
+
+static __hwloc_inline int
+hwloc_utils_cpuset_format_sscanf(hwloc_bitmap_t set, const char *string, enum hwloc_utils_cpuset_format_e cpuset_format)
+{
+  if (cpuset_format == HWLOC_UTILS_CPUSET_FORMAT_UNKNOWN) {
+    /* If user doesn't enforce a format, try to guess.
+     * There is some ambiguity if a list of singleton like 1,3,5 is given,
+     * it may be parsed as 0x1,0x3,0x5 (hwloc) or 1-1,3-3,5-5 (list).
+     * Assume list first, then hwloc, then taskset.
+     */
+    if (hwloc_strncasecmp(string, "0x", 2) && strchr(string, '-'))
+      cpuset_format = HWLOC_UTILS_CPUSET_FORMAT_LIST;
+    else if (strchr(string, ','))
+      cpuset_format = HWLOC_UTILS_CPUSET_FORMAT_HWLOC;
+    else
+      cpuset_format = HWLOC_UTILS_CPUSET_FORMAT_TASKSET;
+  }
+
+  switch (cpuset_format) {
+  case HWLOC_UTILS_CPUSET_FORMAT_HWLOC:
+    return hwloc_bitmap_sscanf(set, string);
+    break;
+  case HWLOC_UTILS_CPUSET_FORMAT_LIST:
+    return hwloc_bitmap_list_sscanf(set, string);
+    break;
+  case HWLOC_UTILS_CPUSET_FORMAT_TASKSET:
+    return hwloc_bitmap_taskset_sscanf(set, string);
+    break;
+  default:
+    /* HWLOC_UTILS_CPUSET_FORMAT_SYSTEMD input not supported */
+    abort();
+  }
 }
 
 static __hwloc_inline unsigned long
@@ -793,7 +1132,13 @@ hwloc_utils_parse_topology_flags(char * str) {
     HWLOC_UTILS_PARSING_FLAG(HWLOC_TOPOLOGY_FLAG_INCLUDE_DISALLOWED),
     HWLOC_UTILS_PARSING_FLAG(HWLOC_TOPOLOGY_FLAG_IS_THISSYSTEM),
     HWLOC_UTILS_PARSING_FLAG(HWLOC_TOPOLOGY_FLAG_THISSYSTEM_ALLOWED_RESOURCES),
-    HWLOC_UTILS_PARSING_FLAG(HWLOC_TOPOLOGY_FLAG_IMPORT_SUPPORT)
+    HWLOC_UTILS_PARSING_FLAG(HWLOC_TOPOLOGY_FLAG_IMPORT_SUPPORT),
+    HWLOC_UTILS_PARSING_FLAG(HWLOC_TOPOLOGY_FLAG_RESTRICT_TO_CPUBINDING),
+    HWLOC_UTILS_PARSING_FLAG(HWLOC_TOPOLOGY_FLAG_RESTRICT_TO_MEMBINDING),
+    HWLOC_UTILS_PARSING_FLAG(HWLOC_TOPOLOGY_FLAG_DONT_CHANGE_BINDING),
+    HWLOC_UTILS_PARSING_FLAG(HWLOC_TOPOLOGY_FLAG_NO_DISTANCES),
+    HWLOC_UTILS_PARSING_FLAG(HWLOC_TOPOLOGY_FLAG_NO_MEMATTRS),
+    HWLOC_UTILS_PARSING_FLAG(HWLOC_TOPOLOGY_FLAG_NO_CPUKINDS)
   };
 
   return hwloc_utils_parse_flags(str, possible_flags, (int) sizeof(possible_flags) / sizeof(possible_flags[0]), "topology");
@@ -857,6 +1202,7 @@ hwloc_utils_parse_local_numanode_flags(char *str) {
   struct hwloc_utils_parsing_flag possible_flags[] = {
     HWLOC_UTILS_PARSING_FLAG(HWLOC_LOCAL_NUMANODE_FLAG_LARGER_LOCALITY),
     HWLOC_UTILS_PARSING_FLAG(HWLOC_LOCAL_NUMANODE_FLAG_SMALLER_LOCALITY),
+    HWLOC_UTILS_PARSING_FLAG(HWLOC_LOCAL_NUMANODE_FLAG_INTERSECT_LOCALITY),
     HWLOC_UTILS_PARSING_FLAG(HWLOC_LOCAL_NUMANODE_FLAG_ALL)
   };
 
